----------------------------------------------------------------------------------
-- Engineer: Ralph Hnsel, 22.05.2011
-- Module Name: BWSv4_COLOR
-- Taktgenerator 20Mhz
-- Anpassung an COLOR-BWS: Heiko Poppe, 21.08.2011
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL ; --notwendig fr die Zhler

entity BWSv4_COLOR is
   Port ( 
			ADR	: in  STD_LOGIC_VECTOR (15 downto 0);
--			DB    : inout  STD_LOGIC_VECTOR (7 downto 0)  ;
			DB    : in  STD_LOGIC_VECTOR (7 downto 0)  ;
			MREQ  : in  STD_LOGIC;
			WR    : in  STD_LOGIC;
			RD    : in  STD_LOGIC;
			Reset : in  STD_LOGIC;
			IORQ  : in  STD_LOGIC;
			MEMDI : in  STD_LOGIC;		-- L wenn Speicher deaktiviert 
			MEMDI_AC1: in  STD_LOGIC;	-- L wenn Modul1 deaktiviert (damit AC1 Speicher)
											   --   dieses Signal ist auch das "CPM ein" Signal vom AC1 2010
			RAMADR	 : out    STD_LOGIC_VECTOR (10 downto 0); -- Adresse AusleseRAM (links)
			RAMDB		 : inout  STD_LOGIC_VECTOR (7 downto 0) ; -- 
			CSRAM		 : out STD_LOGIC;								-- CS Signal fr RAM (rechts) CPU-Seite
			CSFARBRAM : out STD_LOGIC;								-- CS Signal fr FARBRAM (rechts) CPU-Seite
			CSREADRAM : out STD_LOGIC;								-- CS Signal fr RAM (links)  AusleseRAM
         ZGADR		 : out STD_LOGIC_VECTOR (2 downto 0);    -- Adresse A0..A2 ZG-Eprom
         ZG_Out    : in  STD_LOGIC_VECTOR (5 downto 0);    -- Ausgnge ZG-Eprom
			ZGADR11   : out STD_LOGIC;	-- Umschaltung ZG ACC<-->SCCH Modus
         Video_out : out STD_LOGIC; -- VideoSignal        negiert (wegen Treiber)
			DTVDU		 : inout STD_LOGIC; -- Dunkeltastung Bildraender
			HK_Sync 	 : out STD_LOGIC; -- HK SynchronImpuls  negiert (wegen Treiber)
			VK_Sync 	 : out STD_LOGIC; -- VK SynchronImpuls  negiert (wegen Treiber)
			Takt_in 	 : in  STD_LOGIC; -- 20 Mhz Eingangstakt
			CSD35_36	 : out STD_LOGIC	-- Datenuebernahme Latch aus Farb-RAM
			);

end BWSv4_COLOR;

architecture VERHALTEN of BWSv4_COLOR is
	signal   VDCLK       : STD_LOGIC :=	'0'							   ;-- VDCLK = Punkttakt 20Mhz
	signal	QD19			: UNSIGNED (2 downto 0) := (others => '0'); -- Ausgang D19 6:1 Taktteiler
	signal	QD41_42		: UNSIGNED (6 downto 0)	:= (others => '0'); -- Ausgang D41 und 42 Zeichenzhler
	signal	QTFT			: UNSIGNED (0 downto 0) := (others => '0'); -- Ausgang QTFT 2:1 Teiler fr TFT Modus
	signal	QD43_44_20	: UNSIGNED (8 downto 0)	:= (others => '0'); -- Ausgang D43,D44,D20 Zhler
	signal   StartZeile  : STD_LOGIC := '0' ; --Signal 1 Start neue Zeile
	signal	BWS_Zugriff : STD_LOGIC := '0' ; --Signal 1 fr Zugriff der CPU auf BWS
	signal	BWS_aus		: STD_LOGIC := '1' ; --Signal 0 wenn BWS ausgeschaltet
	signal	D35_36		: STD_LOGIC_VECTOR(7 downto 0) := (others => '0') ; -- Latch zum Zeichen speichern
	signal   SR_ZG_Out   : STD_LOGIC_VECTOR (5 downto 0) := (others => '0') ; -- Ausgnge Schieberegister ZG
	signal	DG13			: STD_LOGIC :=	'0'  ; -- Dunkeltastung = 1 bei Strahlrcklauf
	signal 	BWS_INV		: STD_LOGIC := '0'  ; -- BWS INVERS Umschaltung
	signal	Zei_INV		: STD_LOGIC := '0'  ; -- Zeichen INVERS Umschaltung
	signal	SR_Ladetakt : STD_LOGIC := '0'  ; -- Takt fr Laden Schieberegister Invers
	signal	FF_INV_EIN  : STD_LOGIC_VECTOR(1 downto 0) := (others => '0') ; -- FlipFlop INVERS Umschaltung
	signal	FF_INV_AUS  : STD_LOGIC_VECTOR(1 downto 0) := (others => '0') ; -- FlipFlop INVERS Umschaltung
   signal 	IO5_Latch   : STD_LOGIC := '0'  ; -- PIO Port B ZG Umschaltung
   signal 	IO11_Latch  : STD_LOGIC_VECTOR(7 downto 0) := (others => '0') ; -- LatchIO11 Bit1=H Bildschirm INVERS
	signal	IO12_Latch	: STD_LOGIC := '0'  ; -- Umschaltung Zeichen-RAM / Farb-RAM

-- ***********************************************************************************
begin
-- hier wird das zentrale Taktsignal fr alle Prozesse definiert, 

	VDCLK		<= Takt_In 	;							-- bei 20Mhz Takt hier Takt_in 

-- IO5 Latch fngt Ausgaben an PIO Port B ab (ZG Umschaltung)
process(VDCLK)
begin
	if	rising_edge(VDCLK) then 
		if	(ADR(7 downto 0) = x"05" and IORQ = '0' and WR = '0') -- IO Adresse PIO1 Port B
			then IO5_Latch  <= DB(3)       ;-- Bit 3 (PIO 1 PortB) ZG Modeumschaltung
			else IO5_Latch  <= IO5_Latch	 ;-- Zustand behalten
		end if;	
	end if;
end process;

-- Umschaltung Zeichengenerator ACC <==> SCCH 
   ZGADR11		<=	ADR(11) when IO11_Latch(7)='1'   -- ADR11 fr ZG Lesen/Schreiben
											else IO5_Latch 	; -- ZG-Umschaltung

-- IO11H Register mit 8 Bit Breite (DB7..0)
process(VDCLK)
begin
	if rising_edge(VDCLK) then
		if Reset = '0' then IO11_Latch  <= (others => '0') ;	-- Reset auswerten
			else
				if (ADR(7 downto 0) = x"0A" and IORQ = '0' and WR = '0')	-- IO Adresse hier 0Ah, sonst 11H
					then IO11_Latch  <= DB(7 downto 0) ;						-- Latch vom DB laden
					else IO11_Latch  <= IO11_Latch 	  ;						-- Zustand behalten
				end if;
		end if;
	end if;
end Process;

-- IO12H Register mit 1 Bit Breite (DB0) zur Umschaltung Zeichen-RAM (0) / Farb-RAM (1)
process(VDCLK)
begin
	if	rising_edge(VDCLK) then 
		if Reset = '0' then IO12_Latch  <= '0' ;	-- Reset auswerten
			else
				if	(ADR(7 downto 0) = x"09" and IORQ = '0' and WR = '0') -- IO Adresse hier 09h, sonst 12H
					then IO12_Latch  <= DB(0)       ;-- Bit 0: 0 = Zeichen-RAM, 1 = Farb-RAM
					else IO12_Latch  <= IO12_Latch  ;-- Zustand behalten
				end if;
		end if;	
	end if;
end process;
	
-- Signal BWS_aus bilden, immer wenn der BWS mittels MEMDI deaktiviert wird
	BWS_aus	<=	'1'	when	MEMDI_AC1 = '0' 		-- Modul 3 Zugriff
								or 
									MEMDI ='1'				-- CPM Zugriff
								else '0' ; 

-- Daten vom Farb-RAM ins Latch einlesen
process (VDCLK)
begin
	if falling_edge(VDCLK) then
		if QD19 = 0							-- fuer Timing Zeichen / Hintergrund auf 0
			then	CSD35_36	<=	'1' 	;
			else  CSD35_36 <= '0'    ; -- Taktsignal fr Latch 74LS374 L/H speichert
		end if;
	end if;
end process;

-- Daten vom Zeichen-RAM ins Latch einlesen
process(VDCLK)
begin
	if rising_edge(VDCLK) then
		if QD19 = 0 then 	D35_36 <= RAMDB 		; -- Zeichen einlesen
							else D35_36 <= D35_36	; -- Latch bilden
		end if;
	end if;
end process;
	
-- Daten vom Zeichengenerator ins Schieberegister SR_ZG_Out holen
ZG_Out_ausschieben: --ZG Daten aus D38&39=SR_ZG_Out(0) rausschieben
process(VDCLK)  -- Zeichentakt
	begin
	if falling_edge(VDCLK) then			-- gendert von rising_edge auf falling_edge
      if QD19 = 0 then  SR_ZG_Out(5 downto 0) <= ZG_Out(5 downto 0) ; -- 0 parallel laden      
					 	 else SR_ZG_Out <= '0' & SR_ZG_Out(5 downto 1) ;  	 -- serial schieben
		end if;
   end if;
end process ZG_Out_ausschieben; -- fertig Videodaten ausschieben

-- hier zuerst Adressdekodierung & Zugriff selektieren 1000H..17FFH (+RW Mode)
 	BWS_Zugriff <= '1' when ADR(15 downto 12) =x"1" -- Adresse 1xxxH 
								and ADR(11)='0' 				-- Adresse 1000H..17FFH
								and MREQ='0' 							-- Speicherzugriff
								and BWS_aus = '0'						-- wenn BWS aktiv sein darf
								else '0' ; 						      -- BWS auslesen

	CSRAM       <= '0' when BWS_Zugriff = '1' and IO12_Latch = '0' else '1' ;

	CSFARBRAM   <= '0' when BWS_Zugriff = '1' and IO12_Latch = '1' else '1' ;

	CSREADRAM   <= '0' when IO11_Latch(4) = '0' -- AusleseRAM und OE ZG deaktivieren
								else '1' ;				  -- AusleseRAM und EEPROM aktivieren

-- RAM Adressen bilden ------------------------------------------------
	RAMADR(5 downto 0)  <= not STD_LOGIC_VECTOR(QD41_42(5 downto 0)) ;
	RAMADR(10 downto 6) <= not STD_LOGIC_VECTOR(QD43_44_20(7 downto 3)) ;

-- ZG Adresse erzeugen
	ZGADR(2 downto 0)  <= ADR(2 downto 0) when IO11_Latch(7) = '1'  		-- ADR ZG Lesen Schreiben
								else STD_LOGIC_VECTOR(QD43_44_20(2 downto 0)) ; -- Bits 0..2 D43 Zeichen

-- INVERS Schieberegister
SR_INV_EIN: -- SR INVERS
process(SR_Ladetakt)  -- 
	begin
	if rising_edge(SR_Ladetakt) then
     if D35_36 = x"11" then FF_INV_EIN(1 downto 0) <= "11" ;  -- parallel laden #11 erkannt
								else
								FF_INV_EIN(1 downto 0) <= FF_INV_EIN(0) & '0' ; -- Bit serial links schieben
		end if;
  end if;
end process SR_INV_EIN; -- fertig 

SR_INV_AUS: -- SR INVERS
process(VDCLK)  --   SR_Ladetakt
	begin
	if falling_edge(VDCLK) then
      if D35_36 = x"10" then FF_INV_AUS(1 downto 0) <= "11" 			;  -- parallel laden #10 erkannt
								else
								FF_INV_AUS(1 downto 0) <= FF_INV_AUS(0) & '0' ; -- Bit serial links schieben
		end if;
   end if;
end process SR_INV_AUS; -- fertig 


-- Ladetakt fuer INVERS-Schieberegister, bestimmt den Start von INVERS, Takt fr FF_INV schieben
	SR_Ladetakt <= '1' when QD19 = 0 and VDCLK = '1' else '0' ;


-- INVERS Modus 10H/11H 
	Zei_INV	 <= '1'	when	FF_INV_EIN(1)	='1'		  -- D35_36 = x"11"	-- InversModus ein
								else '0' when 
											 FF_INV_AUS(1) ='1' -- D35_36 = x"10"  -- InversModus aus 
										 or
										 QD41_42 = 70			  -- InversModus aus am Zeilenende
								else Zei_INV ;
--  Zei_INV <= '0' ;



-- INVERS-Auswertung	
	BWS_Inv     <= IO11_Latch(1) when Zei_INV ='0'	   	 -- Umschaltung INVERS
											else not IO11_Latch(1) ; -- Umschaltung Invertieren	

-- Hinweis: durch die Treiber nach dem CPLD muss die Invertierung des Signales beachtet werden
	Video_Out <= '1' when DTVDU = '1' 		-- Dunkeltastung
							else SR_ZG_out(0) when BWS_INV = '1' 	    -- INVERS oder NICHT INVERS
													 else not SR_ZG_out(0) ; -- Ausgang Schieberegister ;

-- Signal DG13 bilden, es stellt die Zeit des Strahlrcklaufes dar
process(VDCLK)
begin
	if	falling_edge(VDCLK) then
				if    QD41_42 = 64 and QD19 = 0 then DG13 <= '1' ; -- Dunkeltastung
				elsif QD41_42 = 0 and QD19 = 0 then DG13 <= '0' ;  -- Anzeige aktiv
				end if;
	end if;
end process;

-- Signal DTVDU bilden, Dunkeltastung fr die Rnder und Austastlcken
	DTVDU	<= '1'   when 	DG13 = '1' 			--	Ausgang FlipFlop DG13
							or	QD43_44_20(8)='1'	--	invertierter D20 
						  else '0' ;			


-- *** ab hier Zhlerkette ********************************************************
-- Hinweis: Die Zhlerkette erzeugt ein SVGA-Signal 800x600 bei 60 Hz 

-- D19 (Basis ist VDCLK) mit 20Mhz
process(VDCLK)     -- Takt wird durch 6=6 Zeilen je Zeichen geteilt 
  begin
    if rising_edge(VDCLK) then 
		if QD19 < 5 then  		-- zhlt 0..5
		  QD19    <= QD19 + 1 ;	-- zhlen
		else
		  QD19    <= "000"; 		-- rcksetzen
			end if ; 
    end if;
  end process; 	

-- Spaltenzhler D41 und D42 Zhlt Zeichen 1-64, dazu Leerrume und Sync-Signal
D41_42: 
process(VDCLK)
	begin
		if rising_edge(VDCLK) then
			if QD19 = 3 then					   -- Freigabe Zhler
				if QD41_42   < 87 then			-- bis 87 zhlen
					QD41_42   <= QD41_42 + 1 ; -- zhlen
				else
					QD41_42   <= "0000000"   ; 
				end if ; 
			end if ;
		end if ;
end process D41_42;				

TFT_Zeile2: 	-- dieser 2:1 Teiler sorgt dafr, dass jede Zeile 2x geschrieben wird
process(VDCLK)
	begin
		if rising_edge(VDCLK) then
		if QD41_42 = 83 and QD19 = 3 then	-- Freigabe Zhler	
				if QTFT   = 0 then				-- bis 1 zhlen
					QTFT   <= QTFT + 1 ; 		-- zhlen
				else
					QTFT   <= "0"   ; 
				end if ; 
			end if ;
		end if ;
end process TFT_Zeile2;				

-- Zeilenzhler D43 & D44 & D20 zhlt Zeilen 0 bis 313
D43_44_20:
process(VDCLK)
begin
	if rising_edge(VDCLK) then 
		if QD41_42 = 83 and QD19 = 3 and QTFT = 1	then	   	 -- Freigabe Zhler	
			if QD43_44_20 < 313 then 									 -- bis 313
										QD43_44_20 <= QD43_44_20 + 1 ; -- zhlen
				else
										QD43_44_20 <= "000000000" 	  ; -- rcksetzen
			end if ;
		end if;
	end if;	
end process D43_44_20;
-- *** BWS Zhlerkette --Ende----------------------------------------------

-- ******** Erzeugung HK Synchronimpuls aus Zhlerkette
-- HK Impuls startet bei Zhlerstand 71 von QD41_42 und ist 11 Takte lang
process(VDCLK)
begin
	if rising_edge(VDCLK) then
									if     QD41_42 = 69 then -- Start HK Impuls
											HK_sync <= '1';
									elsif  QD41_42 = 80 then -- Ende HK Impuls
											HK_sync <= '0';	
									end if;
	end if;
end process;
-- ***Ende HK_Sync Impuls ****************************************************************

-- ******** Erzeugung VK Synchronimpuls aus Zhlerkette
-- VK Impuls startet bei Zhlerstand 278 von QD43_44_20 und ist 2 Takte lang
process(VDCLK)
begin
	if rising_edge(VDCLK) then
									if 	QD43_44_20 	= 278 then -- Start VK Impuls
												VK_sync <= '1';
									elsif QD43_44_20 	= 280 then -- Ende VK Imuls
												VK_sync <= '0';
									end if;
	end if;
end process;
-- ***Ende VK_Sync Impuls ****************************************************************
end VERHALTEN;
